ESeller = AceLibrary("AceAddon-2.0"):new("AceConsole-2.0", "AceEvent-2.0", "AceDB-2.0", "AceDebug-2.0", "AceHook-2.1", "FuBarPlugin-2.0");
local abacus = AceLibrary("Abacus-2.0");
local L = AceLibrary("AceLocale-2.2"):new("EnchantingSell")
local ESUtil = AceLibrary("EnchantingSell-Util-1.0")

ESeller:RegisterDB("EnchantingSellerDB", "EnchantingSellerDBPerChar", "char");
ESeller:RegisterDefaults("char", {
    UseTooltips = true,
})
ESeller:RegisterDefaults("realm", {
    DBVersion = 0,
    EnchantList={},
    EnchantMaterials={},
    Enchanters={},
})

--fubar plugin options
ESeller.hasIcon = true;
ESeller.hasNoColor = true;
ESeller.hasNoText = true;
ESeller.cannotDetachTooltip = true;
ESeller.defaultPosition = "MINIMAP";
ESeller.defaultMinimapPosition = 155;

--constants
ESellerConstants={};
ESellerConstants.DBVersion = 30005;

--runtime variables
local ES_CurrentPlayer={};

function ESeller:OnInitialize()
	--Setup our chat command interface
	self:RegisterChatCommand({ "/enchantingsell", "/enchantingseller", "/es" },
		{
			type = "group",
			args = {
				CreateDefaultDB = {
					type = "execute",
					name = "CreateDefaultDB",
					desc = "create a snapshot of your known enchants and materials",
			    	      	func = function() self:CreateDefaultDB(); end,
				},
				tooltips = {
					type = 'toggle',
					name = "tooltips",
					desc = "Toggle material counts in tooltips",
					get = function()
						return self.db.char.UseTooltips;
					end,
					set = function(v)
						self:SetUseTooltips(v);
						self:UseTooltips_Update();
					end,
					map = { [false] = "Disabled", [true] = "Enabled" },
				}

--				debug = {
--					type = "text",
--					name = "additem",
--					desc = "add an item to the database",
--					usage = "<itemlink>",
--					get = false,
--			    	      	set = function(v) self:AddItem(v); end,
--				},
--				debugLevel = {
--					type = "text",
--					name = "debugLevel",
--					desc = "change debug level.  1=INFO, 2=TRACE",
--					usage = "<level>",
--					get = false,
--			    	      	set = function(v) self:SetDebugLevel(tonumber(v)); end,
--				},
			}
		},
		"ENCHANTINGSELL"
	);

	-- Hook in new tooltip code
	if (IsAddOnLoaded("EnhTooltip") and IsAddOnLoaded("Stubby")) then
		Stubby.RegisterFunctionHook("EnhTooltip.AddTooltip", 500, ES_HookTooltip)
	end

	self:SecureHook("CraftFrameFilterDropDown_OnClick")
	self:SecureHook("CraftFrame_Update")

	ES_CurrentPlayer = {UnitName("player"), GetCVar("realmName")};

end

function ESeller:OnEnable()
	self:SetDebugging(false);

	-- events
	self:RegisterOurEvents();

	--self:CreateFrames();
	--for now
	ES_tester.CreateFrames();
	
	--register Escape to close our frame
	tinsert(UISpecialFrames,"ES_tester.EFrame");

	local isEnchanter = ESUtil:IsEnchanter();
	if (isEnchanter and not self.db.realm.Enchanters[ES_CurrentPlayer[1]]) then
		self.db.realm.Enchanters[ES_CurrentPlayer[1]]=true
	end

end

function ESeller:RegisterOurEvents()
	self:RegisterEvent("BAG_UPDATE", "TempEventHandler");
	self:RegisterEvent("CRAFT_UPDATE", "TempEventHandler");
	self:RegisterEvent("BANKFRAME_OPENED", "TempEventHandler");
	self:RegisterEvent("BANKFRAME_CLOSED", "TempEventHandler");
	self:RegisterEvent("SKILL_LINES_CHANGED", "TempEventHandler");
end

function ESeller:TempEventHandler()
end

function ESeller:CraftFrame_Update()
	CraftFrameAvailableFilterCheckButton:Hide();
	CraftFrameEditBox:Hide();
	CraftFrameFilterDropDown:Hide();
end

function ESeller:CraftFrameFilterDropDown_OnClick()
	self:Print(L["EnchantingSeller has prevented you from changing this filter.  Changing this filter can have an unexpected result while using EnchantingSeller."]);
end

--handle the fubar onclick
function ESeller:OnClick()
	--left click toggle
	self:Launch()

	--right click for options
end

function ESeller:Launch()
	local isShown = ES_tester.Toggle_EnchantingSell();

	--only continue if we are launching, no need to perform these if we are closing.
	if (isShown) then
		self:CheckDatabaseVersion();

		local enchantList = {};
		ESUtil:RetrieveCraftList(enchantList);

		--build extra data.
		ESUtil:AddBonusInfo(enchantList);

		self.db.realm.EnchantList = enchantList;
		ES_tester.RefreshEnchantTable();

--	local isEnchanteur = ESUtil:LaunchFunctionInCraftSpellFrame(
--		function () 
--			if ((not EnchantingSell_Config.EnchantorPlayerSelected) or ESell_Player_IsEq(EnchantingSell_Config.EnchantorPlayerSelected, EnchantingSell_CourantPlayer)) then
--				if not EnchantingSell_Config.EnchantorPlayerSelected then
--					ESell_Reagent_DeleteAllplayerCount();
--					EnchantingSell_Config.EnchantorPlayerSelected = EnchantingSell_CourantPlayer;
--				end
--				ESell_Enchante_UpdateData();
--				ESell_Reagent_UpdateData();
--				ESell_Enchante_UpdateAllPrice();
--			end
--		end
--	);
--	ESell_Reagent_UpdateNbInBag();
	end

end

function ESeller:CreateDefaultDB()
	-- in order to create a default database that is stored in the saved variables file.
	for j, enchanteTable in ipairs(EnchantingSell_ListEnchante) do
		enchanteTable["IdOriginal"] = nil;
		enchanteTable["IsKnow"] = nil;
		enchanteTable["Price"] = nil;
		enchanteTable["PriceNoBenef"] = nil;
		enchanteTable["TypePrice"] = nil;
		enchanteTable["Reagents"]["Etat"] = -2;
		newEnchante = false;
	end
	for i, reagent in ipairs(EnchantingSell_ListComponant) do
		reagent["Description"] = nil;
		reagent["ByPlayer"] = nil;
		reagent["IsUse"] = nil;
	end
	ESellSvgEnchante ={};
	ESellSvgEnchante.Componantes = EnchantingSell_ListComponant;
	ESellSvgEnchante.Enchantes = EnchantingSell_ListEnchante;

	self:Print("Bd Create close the game and copie Bd in Localization file");
	return;
end



function ES_HookTooltip(funcVars, retVal, frame, name, link, quality, count)
	if (ESeller:UseTooltips()) then
--		local idReagent = ESell_Reagent_getId(name);

--		if idReagent then
--			local nbInBag, nbInBank, nbInReroll = ESell_Reagent_getCount(idReagent);
--			local money=abacus:FormatMoneyFull(ESell_Reagent_getPrice(idReagent), true);
--
--			EnhTooltip.AddSeparator()
--			EnhTooltip.AddLine(ENCHANTINGSELL_TOOLTIPADD_TITLE..":", nil, false);
--			EnhTooltip.LineColor(0,1,1);
--			EnhTooltip.AddLine(ENCHANTINGSELL_TOOLTIPADD_ONME..": "..nbInBag, nil, false);
--			EnhTooltip.LineColor(1,0.3,1);
--			EnhTooltip.AddLine(ENCHANTINGSELL_TOOLTIPADD_INBANK..": "..nbInBank, nil, false);
--			EnhTooltip.LineColor(1,0.3,1);
--			EnhTooltip.AddLine(ENCHANTINGSELL_TOOLTIPADD_OTHERPLAYER..": "..nbInReroll, nil, false);
--			EnhTooltip.LineColor(1,0.3,1);
--			EnhTooltip.AddLine(ENCHANTINGSELL_TOOLTIPADD_PRICEUNITE..": "..money, nil, false);
--			EnhTooltip.LineColor(1,0.3,1);
--		end
	end

end

function ESeller:UseTooltips_Update()
	--need to hook tooltip
	if (self:UseTooltips()) then
		if (IsAddOnLoaded("EnhTooltip") and IsAddOnLoaded("Stubby")) then
			Stubby.RegisterFunctionHook("EnhTooltip.AddTooltip", 500, ES_HookTooltip)
		end
	else
		if (IsAddOnLoaded("EnhTooltip") and IsAddOnLoaded("Stubby")) then
			Stubby.UnregisterFunctionHook("EnhTooltip.AddTooltip", ES_HookTooltip)
		end
	end
end

function ESeller:UseTooltips()
	return self.db.char.UseTooltips;
end

function ESeller:SetUseTooltips(value)
	self.db.char.UseTooltips=value;
end

-- this function will compare the current DB version, and the users DB version and see if it needs to be updated.
-- this will only be used when key data has changed, like the occurence in wow patch 2.0.1, when itemlinks were changed.
function ESeller:CheckDatabaseVersion()

	local needsCleaning=false;
	
	if (self.db.realm.DBVersion<ESellerConstants.DBVersion) then
		self:Print(L["EnchantingSeller has detected that your database is outdated.  It will be reset now."]);

		self.db.realm.EnchantList={};
		self.db.realm.EnchantMaterials={};
		self.db.realm.DBVersion=ESellerConstants.DBVersion;
	end
	
end


